﻿using EPiServer;
using EPiServer.Core;
using EPiServer.DataAbstraction;
using Frontify.Episerver.Plugin;
using Frontify.Episerver.Plugin.Interfaces;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Json;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using EPiServer.ServiceLocation;

namespace AlloyTemplates.Frontify
{
    //[ServiceConfiguration(ServiceType = typeof(IFolderResolver))]
    public class ArchiveFolderResolver : IFolderResolver
    {
        private readonly IFrontifySettings _frontifySettings;
        private readonly IContentRepository _contentRepository;
        private readonly IdentityMappingService _identityMappingService;
        private readonly string _projectApiUrl = ConfigurationManager.AppSettings["Frontify:ProjectApiUrl"];

        public ArchiveFolderResolver(IFrontifySettings frontifySettings, IContentRepository contentRepository,
            IdentityMappingService identityMappingService)
        {
            _frontifySettings = frontifySettings;
            _contentRepository = contentRepository;
            _identityMappingService = identityMappingService;
        }

        public ContentReference GetAssetFolder(AssetInformation assetInformation, FrontifyItemMetadata dataItem)
        {
            if (_frontifySettings.RootFolderId == null)
            {
                throw new NullReferenceException(
                    "You haven't specified FrontifySettings for RootFolderId, please implement or set Frontify:RootFolderId in appsettings web.config.");
            }

            if (string.IsNullOrEmpty(_projectApiUrl))
                return _frontifySettings.RootFolderId;

            var metadata = dataItem.MetadataAsJson;
            int parentfolderId = 0;
            string project_slug = null;
            string project_id = null;
            ContentReference assetFolderContentReference = _frontifySettings.RootFolderId;
            if (metadata.ContainsKey("set") && int.TryParse(metadata["set"]?.ToString(), out parentfolderId))
            {
                parentfolderId = parentfolderId;
            }

            if (metadata.ContainsKey("project_slug"))
            {
                project_slug = metadata["project_slug"];
                project_id = metadata["project_id"]?.ToString();
            }

            #region check if project has folder

            string proj_uri = "epi://frontify.com/projects/" + project_id;
            var existingMapping = _identityMappingService.Get(new Uri(proj_uri));
            if (existingMapping == null)
            {
                //create the project folder as parentfolder
                IContent folder = CreateFolder(_frontifySettings.RootFolderId, project_slug);
                _identityMappingService.MapContent(new Uri(proj_uri), folder);
                assetFolderContentReference = folder.ContentLink;
            }
            else
            {
                assetFolderContentReference = existingMapping.ContentLink;
            }

            #endregion      

            if (parentfolderId > 0) //if asset is in subfolder
            {

                string uri = proj_uri + "/" + parentfolderId;
                existingMapping = _identityMappingService.Get(new Uri(uri));

                if (existingMapping == null)
                {
                    List<string> segments = new List<string>();
                    //Get folder structure
                    IEnumerable<KeyValue> folders = GetRecursiveFolders(project_id, parentfolderId);

                    foreach (KeyValue val in folders)
                    {
                        assetFolderContentReference = GetFolderContentRefOrCreate(assetFolderContentReference,
                            val.name, int.Parse(val.id), proj_uri);
                    }
                    return assetFolderContentReference;
                }

                return existingMapping.ContentLink;
            }

            return assetFolderContentReference;
        }

        private IEnumerable<KeyValue> GetRecursiveFolders(string project_id, int parentfolderId)
        {
            List<KeyValue> keyValues = new List<KeyValue>();
            JsonValue jsonFolder =
                GetFolderPath($"{_projectApiUrl}{project_id}/{parentfolderId}", int.Parse(project_id))
                    .Result;

            if (jsonFolder["folder"]["parent"])
                keyValues.AddRange(GetRecursiveFolders(project_id, jsonFolder["folder"]["parent"]));

            keyValues.Add(new KeyValue { id = jsonFolder["folder"]["id"]?.ToString(), name = jsonFolder["folder"]["name"] });
            return keyValues;
        }

        private IContent CreateFolder(ContentReference parentfolderId, string foldername)
        {
            //var childFolders = _contentRepository.GetChildren<ContentFolder>(parentfolderId);
            //var categoryFolder = childFolders.FirstOrDefault(f => String.Equals(f.Name, foldername, StringComparison.OrdinalIgnoreCase));
            //if (categoryFolder == null)
            //{
            var newFolder = _contentRepository.GetDefault<ContentFolder>(parentfolderId);
            newFolder.Name = foldername;
            _contentRepository.Save(newFolder);
            return newFolder;
            //}
        }

        public async Task<JsonValue> GetFolderPath(string absoluteUrl, int projectId)
        {
            var httpClient = new HttpClient();
            var request = new HttpRequestMessage { RequestUri = new Uri(absoluteUrl) };

            request.Headers.Add("Authorization", $"Bearer {_frontifySettings.GetApiToken(projectId)}");

            var response = await httpClient.SendAsync(request);

            if (response.StatusCode == HttpStatusCode.NotFound)
            {
                throw new FileNotFoundException("Path not found " + absoluteUrl);
            }

            if (response.StatusCode == HttpStatusCode.Unauthorized)
            {
                throw new FileNotFoundException("Path Unauthorized " + absoluteUrl);
            }

            var resultBody = await response.Content.ReadAsStringAsync();
            var parsedJson = JsonValue.Parse(resultBody) as JsonObject;
            //check the success and error param {"success":false,"error":"No assets found"}
            if (parsedJson["success"] != true)
            {
                throw new FileNotFoundException("Path not found " + absoluteUrl);
            }

            return parsedJson;
        }


        private ContentReference GetFolderContentRefOrCreate(ContentReference parentfolder, string foldername, int id, string proj_uri)
        {
            string uri = proj_uri + "/" + id;
            var existingMapping = _identityMappingService.Get(new Uri(uri));
            if (existingMapping == null)
            {
                var folder = CreateFolder(parentfolder, foldername);
                _identityMappingService.MapContent(new Uri(uri), folder);
                return folder.ContentLink;
            }

            return existingMapping.ContentLink;
        }

        public class KeyValue
        {
            public string name { get; set; }
            public string id { get; set; }
        }
    }
}